#include <nds.h>

#define LUT_SIZE (512)
#define LUT_MASK (0x1FF)

void glVertex3f32 (f32 x, f32 y, f32 z) {
	glVertex3v16(f32tov16(x), f32tov16(y), f32tov16(z));
}

void glTranslatef32 (f32 x, f32 y, f32 z) {
  MATRIX_TRANSLATE = x;
  MATRIX_TRANSLATE = y;
  MATRIX_TRANSLATE = z;
}

void glScalef32 (f32 x, f32 y, f32 z) {
  MATRIX_SCALE = x;
  MATRIX_SCALE = y;
  MATRIX_SCALE = z;
}

void glRotatef32 (f32 angle, f32 x, f32 y, f32 z) {
  	f32 v[3];
  	f32 sine = SIN[f32toint(divf32(mulf32(angle ,intof32(LUT_SIZE)),intof32(360))) & LUT_MASK];
  	f32 cosine = COS[f32toint(divf32(mulf32(angle ,intof32(LUT_SIZE)),intof32(360))) & LUT_MASK];
  	f32 cos_off = intof32 (1) - cosine;

	v[0] = x;
	v[1] = y;
	v[2] = z;
 	
 	normalizef32 (v);
 	
	f32 xy = mulf32(v[0], v[1]);
	f32 xz = mulf32(v[0], v[2]);
	f32 yz = mulf32(v[1], v[2]);
	f32 xs = mulf32(v[0], sine);
	f32 ys = mulf32(v[1], sine);
	f32 zs = mulf32(v[2], sine);

  	MATRIX_MULT4x4 = mulf32(mulf32(v[0], v[0]), cos_off) + cosine;
  	MATRIX_MULT4x4 = mulf32(xy, cos_off) + zs;
  	MATRIX_MULT4x4 = mulf32(xz, cos_off) - ys;
  	MATRIX_MULT4x4 = 0;

  	MATRIX_MULT4x4 = mulf32(xy, cos_off) - zs;
  	MATRIX_MULT4x4 = mulf32(mulf32(v[1], v[1]), cos_off) + cosine;
  	MATRIX_MULT4x4 = mulf32(yz, cos_off) + xs;
  	MATRIX_MULT4x4 = 0;
  
  	MATRIX_MULT4x4 = mulf32(xz, cos_off) + ys;
  	MATRIX_MULT4x4 = mulf32(yz, cos_off) - xs;
  	MATRIX_MULT4x4 = mulf32(mulf32(v[2], v[2]), cos_off) + cosine;
  	MATRIX_MULT4x4 = 0;
  
  	MATRIX_MULT4x4 = 0;
  	MATRIX_MULT4x4 = 0;
  	MATRIX_MULT4x4 = 0;
  	MATRIX_MULT4x4 = intof32 (1);
}

void glNormal3f32 (f32 x, f32 y, f32 z) {
	glNormal(NORMAL_PACK(f32tov10(x),f32tov10(y),f32tov10(z)));
}

void glColor3f32 (f32 r, f32 g, f32 b) {
	uint8 red = f32toint(mulf32 (r,intof32(31)));
	uint8 green = f32toint(mulf32 (g,intof32(31)));
	uint8 blue = f32toint(mulf32 (b,intof32(31)));
	GFX_COLOR = (vuint32)RGB15(red, green, blue);
}

void glInit () {
	glNormal3f32 (0,0,intof32(1));
	glMaterialf(GL_AMBIENT, BIT(15) | RGB15(19,19,19));
	glMaterialf(GL_DIFFUSE, BIT(15) | RGB15(28,28,28));
	glMaterialf(GL_SPECULAR, BIT(15) | RGB15(16,16,16));
	glMaterialf(GL_EMISSION, BIT(15) | RGB15(16,16,16));
	glPolyFmt(POLY_ALPHA(31) | POLY_CULL_NONE);
	glColor3f32 (intof32(1),intof32(1),intof32(1));	
}
