#include <nds.h>
#include <nds/arm9/console.h> //basic print funcionality

#include "tris_md2.h"
#include "texture_bin.h"
#include "splash_bin.h"
#include "../md/md.h"
#include "../gl/glRoutines.h"

int main()
{	
	int md_textid;
	int rotateX = 0;
	int rotateY = 0;

	powerON(POWER_ALL);

	videoSetMode(MODE_0_3D);
	videoSetModeSub(MODE_5_2D | DISPLAY_BG3_ACTIVE);  

	irqInitHandler(irqDefaultHandler);
	irqSet(IRQ_VBLANK, 0);

	glViewPort(0,0,255,191);
	
	glClearColor(0,0,0);
	glClearDepth(0x7FFF);

	vramSetBankA(VRAM_A_TEXTURE);
	vramSetBankB(VRAM_B_TEXTURE);
	vramSetBankC(VRAM_C_SUB_BG); 
	
	SUB_BG3_CR = BG_BMP16_256x256;
	
	SUB_BG3_XDX = 1 << 8;
    SUB_BG3_XDY = 0;
    SUB_BG3_YDX = 0;
    SUB_BG3_YDY = 1 << 8;
	
	for(int i = 0; i < 192*256; i++)
	   BG_GFX_SUB[i] = ((u16*)splash_bin)[i] | BIT(15); 

	
	glGenTextures( 1, &md_textid );
	glBindTexture( 0, md_textid );
	glTexImage2D( 0, 0, GL_RGB, TEXTURE_SIZE_128 , TEXTURE_SIZE_128, 0, TEXGEN_TEXCOORD, (u8*)texture_bin );
	
	int jumper = 0;
	int jumper_max = getNumFrames (tris_md2);

	glInit();
	TIMER0_DATA = 0;
	TIMER0_CR = TIMER_DIV_1024;
	while(1)		
	{
		glReset();
	
		//any floating point gl call is being converted to fixed prior to being implemented
		gluPerspective(35, 256.0 / 192.0, 0.1, 40);

		gluLookAt(	0.0, 0.0, 1.0,		//camera possition 
					0.0, 0.0, 0.0,		//look at
					0.0, 1.0, 0.0);		//up
		
		

		glPushMatrix();
				
		glRotateX(rotateX);
		glRotateY(rotateY);
		
		glMatrixMode(GL_TEXTURE);
		glIdentity();
		
		glMatrixMode(GL_MODELVIEW);

	
		//not a real gl function and will likely change
		glPolyFmt(POLY_ALPHA(31) | POLY_CULL_NONE);

		if(!(KEYS & KEY_UP)) rotateX += 3;
		if(!(KEYS & KEY_DOWN)) rotateX -= 3;
		if(!(KEYS & KEY_LEFT)) rotateY += 3;
		if(!(KEYS & KEY_RIGHT)) rotateY -= 3;
	
		glPushMatrix();
		glBindTexture(0, md_textid);
		loadMD2Frame(tris_md2,jumper);	
		while (TIMER0_DATA < 4096) {}
		TIMER0_CR = 0;
		TIMER0_DATA = 0;
		TIMER0_CR = TIMER_DIV_1024;
		glPopMatrix(1);
		glFlush();
		
		jumper++;
		if (jumper >= jumper_max) jumper = 0;
	}

	return 0;
}
